/*   lcd.c  - LCD DIsplay panel handler

  handles:
    KeyPad/encoder/LCD Operation 
    included in main to add LCD/keypad functionality
	
	MAJOR ROUTINES:
void lcd_init(void)- initialize the LCD interface
void lcd_putch(char c)- write a char to the LCD display 
void lcd_putl(char *c)- write a line to the LCD display (and pads)
void lcd_putl2(const char *c)- write to seconds  line to the LCD display (and pads)
void lcd_cursor(void) - turn on cursor
void lcd_goto(unsigned char pos) - Jump to a location on the lcd 
void lcd_clr(void)- CLEAR LCD SCREEN 
void do_keypad(void) - Main Dispatch for keypad keystrokes
unsigned char keypad_press()-  returns 1 if someting is pressed, 0 = none
unsigned char encoder_in(void)- waits for encoder or entry 0 = Nothing waiting, 1 = entry, 2 = exit, 3 = menu, 4 left, 5 right   


    Version      Author   Date           Comments
     1.00           hg.   7/11/00  Initial Development
	 1.01			hg.	  7/4/02   Added Encoder support
*/

#define LCDLIB 1

// only choose one of these - commen tout the other
// the encoder wheel (knob) or the kaypad model 
#define ENCODER 1
// #define KEYPAD 1   // use for keypads

/** PROTOTYPES **/
void lcd_putch(char c);
void lcd_putl(char *c);
void lcd_putl2(const char *c);
void lcd_goto(unsigned char pos);
void lcd_clr(void);
void lcd_cursor(void);
void do_keypad(void);
char read_keypad(void);
void lcd_init(void);
unsigned char keypad_press(void);
unsigned char encoder_in(void);
void handle_keypress(char newkey);


/**** Hardware ports *****/
/* LCD display stuff */
#define LCD_CHARPERLINE 0x40
#define LCD_CMD_DELAY 3		   // time to wait between LCD commands (3 works on LCDS)

#define TOGGLE_LCD_CS()    PTA |= PA_LCD_CS, PTA &= ~PA_LCD_CS  // toggle CS


/* keypad stuff */
#ifdef KEYPAD 
/* keypad lookup stuff */
const char  keylookup[16] = {'1','2','3','U','4','5','6','D','7','8','9','N','C','0','H','E'};
#endif

/* ENCODER Stuff */
#ifdef ENCODER
unsigned char cv,nv;
#endif

// Basic User input stuff
unsigned char user_input_data;	// current action															 
char curpress;
unsigned char lcdtimeout;
unsigned char dbounce;		  		  /* debounce counter */

// USER INPUT DEFINES
#define LEFTDOWN 'D'
#define RIGHTUP 'U'
#define ENTER 'E'
#define EXIT 'C'



/* LCD_INIT - initialize the LCD interface */
void lcd_init(void)
{
volatile int i;

  PTA &= ~PA_LCD_A0;
  PTA &= ~PA_LCD_CS;
  // 3C
  spi_write(0x38,PC_SEL_LCD);  // 2x40 Display
  TOGGLE_LCD_CS();
  delay(LCD_CMD_DELAY);  // delay 10ms for command
  // 3C
  spi_write(0x38,PC_SEL_LCD);  // 2x40 Display
  TOGGLE_LCD_CS();
  delay(LCD_CMD_DELAY);  // delay 10ms for command
  spi_write(0x0F,PC_SEL_LCD);  // Clear & Home
  TOGGLE_LCD_CS();
  delay(LCD_CMD_DELAY);  // delay 10ms for command
  spi_write(0x01,PC_SEL_LCD);  // Display on
  TOGGLE_LCD_CS();
  delay(3);  // delay 10ms for command
  spi_write(0x06,PC_SEL_LCD);  // ursor shift on
  TOGGLE_LCD_CS();
  delay(LCD_CMD_DELAY);  // delay 10ms for command
  spi_write(0x14,PC_SEL_LCD);  // ursor shift on
  TOGGLE_LCD_CS();
  delay(LCD_CMD_DELAY);  // delay 10ms for command
  spi_write(0x02,PC_SEL_LCD);  // Cursor to Home
  TOGGLE_LCD_CS();
  delay(LCD_CMD_DELAY);  // delay 10ms for command
}



/* LCD_PUTCH - write a char to the LCD display */
void lcd_putch(char c)
{
volatile int i;

  PTA |= PA_LCD_A0;
  PTA &= ~PA_LCD_CS;
  spi_write(c,PC_SEL_LCD);  // Cursor to Home
  TOGGLE_LCD_CS();
  //caldelay(100);  // delay 10ms for data
}

/* LCD_PUTL - write a line to the LCD display (and pads)*/
void lcd_putl(char *c)
{
unsigned char q;

  PTA &= ~PA_LCD_A0;
  PTA &= ~PA_LCD_CS;
  spi_write(0x02,PC_SEL_LCD);  //  GOTO 1st Line - home 
  TOGGLE_LCD_CS();
  delay(LCD_CMD_DELAY);  // delay 10ms for command
  q=0;
  while (*c |= '\0')  {
    lcd_putch(*c);
    c++;
	q++;
  }
  while (q < 20) {  // pad spaces
    lcd_putch(' ');
    q++;
  }
}

/* LCD_PUTL2 - write to seconds  line to the LCD display (and pads) */
void lcd_putl2(const char *c)
{
char ch;
unsigned char q;

  lcd_goto(LCD_CHARPERLINE);
  ch = *c;
  q=0;
  while (ch |= '\0')  {
    lcd_putch(ch);
    c++;
	q++;
    ch = *c;
  }
  while (q < 20) {  // pad spaces
    lcd_putch(' ');
    q++;
  }
}

/* LCD_CURSOR - turn on cursor */
void lcd_cursor(void)
{
  PTA &= ~PA_LCD_A0;
  PTA &= ~PA_LCD_CS;
  spi_write(0x10,PC_SEL_LCD);  // Cursor blink
  TOGGLE_LCD_CS();
  delay(LCD_CMD_DELAY);  // delay 10ms for command
}

/* LCD_GOTO - Jump to a location on the lcd */
void lcd_goto(unsigned char pos)
{

  PTA &= ~PA_LCD_A0;
  PTA &= ~PA_LCD_CS;
  spi_write(0x80 | pos,PC_SEL_LCD);  //  GOTO  Line  
  TOGGLE_LCD_CS();
  delay(LCD_CMD_DELAY);  // delay 10ms for command
       
}

/* lcd_clr - CLEAR LCD SCREEN */
void lcd_clr(void)
{

  PTA &= ~PA_LCD_A0;
  PTA &= ~PA_LCD_CS;
  spi_write(1,PC_SEL_LCD);  //  Clear & Home */
  TOGGLE_LCD_CS();
  delay(3);  // delay 10ms for command
}

#ifdef KEYPAD
/* DO_KEYPAD - Main Dispatch for keypad keystrokes  */
void do_keypad(void)
{
char lastread;

  lastread = user_input_data;
  if (lastread != curpress) {
     SLEEP(4);  /* wait for debounce */       
     if (lastread == user_input_data) {
        /* ok check again if still same we got a good one */
        curpress = lastread;
        handle_keypress(curpress);
        lcdtimeout = 80;
        return;
     }
  }
  if (lastread == 0)
    curpress = 0;
}

/* READ_KEYPAD - read the current keypad press and return the decoded value */
char read_keypad(void)
{
unsigned char i;
unsigned char ssel;   /* select bit */
unsigned char sscan;  /* scan result */
unsigned char press;   /* the key being pressed */
unsigned char ii;


  /* scan keypad for press */
  ssel=1;
  for (i=0;i<4;i++) {
      /* walk a one accross the scanns */
      kp_io_reg &= 0xF0;
	  kp_io_reg |= (~ssel & 0x0F);
	  spi_write(kp_io_reg,PC_SEL_KPAD);  // send it out
      sscan = (PTA & PA_KP_MASK);
      if (sscan != PA_KP_MASK) {  /* key pad press - decode */
         ssel = 0x10;
         for (ii=0;ii<4;ii++) {
           if ((ssel & sscan) == 0) {
              /* decoded -     */
              press = ((i * 4) + ii) + 1;
              return(keylookup[press-1]);
           }
           ssel= ssel << 1;
         }
		 ssel=1;  // lost key? not found
      }
      ssel= ssel << 1;
  }
  kp_io_reg &= 0xF0;  // LEave al bits low so we can detect any press
  spi_write(kp_io_reg,PC_SEL_KPAD);  // send it out
  return(0);
}

/* KEYPAD_PRESS -  returns 1 if someting is pressed, 0 = none */
unsigned char keypad_press()
{
unsigned char sscan;  /* scan result */
      sscan = (PTA & PA_KP_MASK);
      if (sscan != PA_KP_MASK)   /* key pad press = something low */
	    return(1);
	  else {
	    curpress = 0;
	    return(0);
	  }
}
#endif

#ifdef ENCODER

/* ENCODER IN - waits for encoder or entry/exit press and returns */
/* 0 = Nothing waiting, 1 = entry, 2 = exit, 3 = menu, 4 left, 5 right  */ 
unsigned char encoder_in(void)
{ 
// current value and new values - watch for transitions  
  
  
  // setup keypad drives for lows
  
    cv = nv;
    // wait for transition
	nv = (PTA & 0xF0);
    if (cv == nv)  // same thing nothing waiting - just get out
	  return(0);
	// debounce
	 if (dbounce)
	   return(0);
    //delay(1);
	//caldelay(100); 
	while (nv != (PTA & 0xF0)) {
      //delay(1); 
  	  //caldelay(100); 
  	  nv = (PTA & 0xF0);
	} 
	dbounce=10;	  
	if (!(nv & 0x10)) 	 // exit press
	  return(EXIT);
	else if (!(cv & 0x10)) // exit release
	  return(0);
	dbounce=100;
	if (!(nv & 0x20)) { 	 // enter
      dbounce=100;	  
	  return(ENTER);
	}
	else if (!(cv & 0x20)) // enter release
	  return(0);
 	dbounce=0;	  
    if ((cv & 0x80) == (nv & 0x80)) {  // bit 6 changed
      if (nv & 0x80) {  // bit 7 is high
        if (nv & 0x40)   // bit 6 just went is high
          return(LEFTDOWN);
        else
  	  	  return(RIGHTUP);
      }
	  else {	// bit 7 low
        if (nv & 0x40)   // bit 6 just went is high
	  	  return(RIGHTUP);
	    else
		  return(LEFTDOWN);
	  }
	}
    else { // bit 7 changed
      if (nv & 0x40) {  // bit 6 is high
       if (nv & 0x80)   // bit 7 just went is high
	  	 return(RIGHTUP);
       else
		 return(LEFTDOWN);
	  }
      else {  // bit 6 low
        if (nv & 0x80)   // bit 7 just went is high
	  	  return(LEFTDOWN);
	    else
  	  	  return(RIGHTUP);
      }
	}

}



#endif

