/*   sclk.c  - Serial Clock (DS1302) handling
   Requires spi.c to be included in main BEFORE sclk.c

  handles:
    Serial Clock Chip (DS1302)
Notes: The DS1302 also stored day of week but DOES NOT calculate it!. 
The DS1302 DOES calculate the end of month and leap years correctly.
Also the DS1302 has 32 bytes of storage that is not used.

The DS1302 has a super cap backup to keep going with no power


void Read_Time(void) -and store values in static memory
void sclk_Write(unsigned char cmd,unsigned char outbyte)- send command byte and data
unsigned char to_BCD(unsigned char dat)- convert tp BCD Nubble style data (used in time chip)
unsigned char from_BCD(unsigned char dat) - convert back from nibble format

time Values decalred in static are 
unsigned char day, month, year,  
unsigned char hour, min, sec; 

    Version      Author   Date           Comments
     1.00           hg.   12/27/00  Initial Development
*/

/** PROTOTYPES **/
unsigned char to_BCD(unsigned char dat);
unsigned char from_BCD(unsigned char dat);
void sclk_Clock(void);
unsigned char sclk_Read(unsigned char cmd);
void sclk_Write(unsigned char cmd,unsigned char outbyte);
void Read_Time(void);

#define SCLKLIB 1

// These are the static location s containing the time and date -
// Use the ReadTime() to refresh
unsigned char day, month, year, dow;  /* current times */
unsigned char hour, min, sec;  /* current times */


/* DS1302 COMMANDS */
#define SCLK_SEC 0x80
#define SCLK_MIN 0x82
#define SCLK_HOUR 0x84
#define SCLK_DAY 0x86
#define SCLK_MONTH 0x88
#define SCLK_YEAR 0x8C
#define SCLK_DOW 0x8A

/* to_BCD - convert tp BCD Nubble style data (used in time chip) */
unsigned char to_BCD(unsigned char dat)
{
  return((dat % 10) + ((dat/10)* 16));
}

/* from_BCD - convert back from nibble format*/
unsigned char from_BCD(unsigned char dat)
{
  return((dat & 0x0F)  + ((dat >> 4) * 10));
}


/* sclk_Clock - clock the SPI clock line */
void sclk_Clock(void)
{

  PTB|=PB_CLK_SCLK;
  PTB&=~PB_CLK_SCLK;
}

/* sclk_Read - send command byte and read data back*/
unsigned char sclk_Read(unsigned char cmd)
{
unsigned char i,q,inbyte;

  cmd|=1;  /* first bit is alway r/w - set to read */
  DDRB|=PB_CLK_IO;  /* turn on the output data bit */
  // Issue (drop) reset
  kp_io_reg |= KIO_CLK_RST;
  spi_write2(kp_io_reg,kp_io_reg2,PC_SEL_KPAD);  // send it out
  q=1;
  for (i=0;i<8;i++) {
    if (q & cmd) PTB |=PB_CLK_IO;  /* set data bit */
    else PTB &=~PB_CLK_IO;			 /* reset data bit */
    sclk_Clock();
    q = q << 1;
  }
  DDRB&=~PB_CLK_IO;  /* turn off the output data bit */
  q=1;
  inbyte = 0;
  for (i=0;i<8;i++) {
    if (PTB & PB_CLK_IO) inbyte |= q;  /* set data bit */
    sclk_Clock();
    q = q << 1;
  }

  PTB |=PB_CLK_IO;
  // Set reset to stop any comm and make DS1302 ignore bogus commands)
  kp_io_reg &= ~KIO_CLK_RST;
  spi_write2(kp_io_reg,kp_io_reg2,PC_SEL_KPAD);  // send it out
  return(inbyte);
}

/* sclk_Write - send command byte and data */
void sclk_Write(unsigned char cmd,unsigned char outbyte)
{
unsigned char i,q;


  cmd&=0xFE;  /* first bit is alway r/w - set to write */
  DDRB|=PB_CLK_IO;  /* turn on the output data bit */
  kp_io_reg |= KIO_CLK_RST;
  spi_write(kp_io_reg,PC_SEL_KPAD);  // send it out
  q=1;
  for (i=0;i<8;i++) {
    if (q & cmd) PTB |=PB_CLK_IO;  /* set data bit */
    else PTB &=~PB_CLK_IO;			 /* reset data bit */
    sclk_Clock();
    q = q << 1;
  }
  q=1;
  for (i=0;i<8;i++) {
    if (q & outbyte) PTB |=PB_CLK_IO;  /* set data bit */
    else PTB &=~PB_CLK_IO;			 /* reset data bit */
    sclk_Clock();
    q = q << 1;
  }

  kp_io_reg &= ~KIO_CLK_RST;		 // terminate xfer
  spi_write(kp_io_reg,PC_SEL_KPAD);  // send it out
  PTB |=PB_CLK_IO;
}

/* Read_Time()  and store values in static memory*/
void Read_Time(void)

{
  /* read current times and store in static */
  sec = from_BCD(sclk_Read(SCLK_SEC));
  min = from_BCD(sclk_Read(SCLK_MIN));
  hour = from_BCD(sclk_Read(SCLK_HOUR));
  month = from_BCD(sclk_Read(SCLK_MONTH));
  day = from_BCD(sclk_Read(SCLK_DAY));
  year = from_BCD(sclk_Read(SCLK_YEAR));
  dow = from_BCD(sclk_Read(SCLK_DOW));
  if ((dow > 7) || (dow < 1)) dow=1;
}


